// coding: utf-8
/*! Created by: Udo Schmal | https://www.gocher.me/ */
(function () {
  'use strict';

  var scale = ((typeof(window.devicePixelRatio) != "undefined") ? window.devicePixelRatio : 1);
  if (!Element.prototype.matches) {
    Element.prototype.matches =
      Element.prototype.msMatchesSelector ||
      Element.prototype.webkitMatchesSelector;
  }

  if (!Element.prototype.closest) {
    Element.prototype.closest = function(s) {
      var el = this;
      do {
        if (Element.prototype.matches.call(el, s)) return el;
        el = el.parentElement || el.parentNode;
      } while (el !== null && el.nodeType === 1);
      return null;
    };
  }
  // create external message
  function externalMessage(el) {
    let txt = el.dataset.external, clk;
    let external = document.createElement('div');
    external.className = 'text';
    let language = document.documentElement.lang || 'en';
    switch (language.substr(0,2)) {
      case 'de':
        txt = ('Externer Inhalt von #, der über einen Klick angeszeigt wird.').replace('#', txt);
        clk = 'Durch diesen Klick werden allerdings auch Daten an Drittplattformen übermittelt!';
        break;
      case 'fr':
        txt = ('Contenu externe de # qui peut être consulté en un clic.').replace('#', txt);
        clk = 'Cependant, ce clic transfère également les données vers des plateformes tierces!';
        break;
      case 'it':
        txt = ('Contenuti esterni da # che possono essere visualizzati con un clic.').replace('#', txt);
        clk = 'Tuttavia, questo clic trasferisce anche i dati a piattaforme di terze parti!';
        break;
      case 'es':
        txt = ('Contenido externo de # que se puede ver con un clic.').replace('#', txt);
        clk = 'Sin embargo, este clic también transfiere datos a plataformas de terceros.';
        break;
      default:
        txt = ('External content from # that can be viewed with a click.').replace('#', txt);
        clk = 'However, this click also transfers data to third-party platforms!';
        break;
    }
    external.appendChild(document.createTextNode(txt));
    external.appendChild(document.createElement('br'));
    external.appendChild(document.createTextNode(clk));
    el.appendChild(external);
    return external;
  }
  // lazy loading image
  function loadImg(el) {
    if (el.dataset.src) {
      if ((el.dataset.scale === 'true') && el.getAttribute('width') && el.getAttribute('height') && (el.dataset.src.indexOf('~') === -1)) {
        let bounding = el.parentNode.getBoundingClientRect();
        let maxWidth = bounding.width;
        let fileName = el.dataset.src.split('.');
        let fileExt = fileName.pop();
        fileName = fileName.join('.');
        if (el.closest('figure.galleryImage')) {
          el.src = fileName + '~' + el.getAttribute('width') + 'x' + el.getAttribute('height') + '.' + fileExt;
        } else if (el.getAttribute('width') > maxWidth*scale) {
          let height = maxWidth * (el.getAttribute('height')/el.getAttribute('width'));
          el.src = fileName + '~' + Math.ceil(maxWidth*scale) + 'x' + Math.ceil(height*scale) + '.' + fileExt;
        } else {
          el.src = fileName + '~' + Math.ceil(el.getAttribute('width')*scale) + 'x' + Math.ceil(el.getAttribute('height')*scale) + '.' + fileExt;
        }
      } else {
        el.src = el.dataset.src;
      }
      el.removeAttribute('data-src');
    }
    if (el.dataset.srcset) {
      el.setAttribute('srcset', el.dataset.srcset);
      el.removeAttribute('data-srcset');
    }
    if (el.dataset.sizes) {
      el.setAttribute('sizes', el.dataset.sizes);
      el.removeAttribute('data-sizes');
    }
  }
  // lazy loading iframe
  function loadIframe(el) {
    if (el.dataset.src) {
      el.src = el.dataset.src;
      el.removeAttribute('data-src');
    }
  }
  // lazy loading video
  function loadVideo(el) {
    if (el.dataset.poster) {
      el.setAttribute('poster', el.dataset.poster);
      el.removeAttribute('data-poster');
    }
    // find best matching source by mediaQuery
    var source = null;
    var media = el.querySelectorAll('source[data-src]');
    if (media.length > 0) {
      for(var i = 0; i < media.length; i++) {
        if (media[i].dataset.media) {
          if (window.matchMedia(media[i].dataset.media).matches) {
            source = media[i];
          }
        }
      }
      if (!source) {
        source = media[0];
      }
    }
    if (source && (!el.active || el.active != source)) {
      source.src = source.dataset.src;
      if (el.active) {
        el.active.removeAttribute('src');
      }
      el.active = source;
    }
    if (el.dataset.controls && el.dataset.controls == 'true') {
      el.setAttribute('controls', '');
      el.removeAttribute('data-controls');
    }
    el.setAttribute('preload', 'auto');
    el.load();
    if (el.dataset.autoplay && el.dataset.autoplay == 'true') {
      el.removeAttribute('data-autoplay');
      el.setAttribute('autoplay', 'autoplay');
      el.autoplay = true;
      el.play();
    }
  }
  // lazy loading external
  function loadExternal(el) {
    var src = el.dataset.src;
    if (el.dataset.external) {
      // create placeholder background image
      if (el.dataset.poster) {
        el.style.backgroundImage = "url('" + el.dataset.poster + "')";
      }/* else {
        el.style.backgroundImage = "url('/media/fallback.jpg')";
      }*/
      // external message
      var msg = externalMessage(el);

      // play button
      var play = document.createElement('div');
      play.className = 'play';
      el.appendChild(play);
      function load(event) {
        msg.classList.add('hidden');
        play.classList.add('hidden');
        var iframe = document.createElement('iframe');
        iframe.setAttribute('allowfullscreen', 'allowfullscreen');
        iframe.setAttribute('loading', 'lazy');
        iframe.src = el.dataset.src;
        el.appendChild(iframe);
        el.style.backgroundImage = 'none';
        play.removeEventListener('click', load);
      }
      play.addEventListener('click', load);
    } else {
      var iframe = document.createElement('iframe');
      iframe.setAttribute('allowfullscreen', 'allowfullscreen');
      iframe.setAttribute('loading', 'lazy');
      iframe.src = el.dataset.src;
      el.appendChild(iframe);
    }
  }
  // lazy loading background image
  function loadBackgroundImage(el) {
    if (el.dataset.backgroundimage) {
      el.style.backgroundImage = "url('" + el.dataset.backgroundimage + "')";
    }
  }
  // lazy loading youtube
  function youtube(el) {
    var player, id;
    // don't send referrer infos to external services
    if (!document.querySelector('meta[name="referrer"]')) {
      let meta = document.createElement('meta');
      meta.name = 'referrer';
      meta.content = 'no-referrer';
      document.head.appendChild(meta);
    }
    // get YouTube video id
    if (el.dataset.src && el.dataset.src.length === 11) {
      id = el.dataset.src;
    } else {
      let src = el.dataset.src;
      let regExp = /^.*((youtube\/)|(v\/)|(\/u\/\w\/)|(embed\/)|(watch\?))\??v?=?([^#&?]*).*/;
      let match = src.match(regExp);
      if (match) {
        id = (match && match[7].length == 11) ? match[7] : src;
      }
    }
    let idx = Math.floor(Math.random() * 6);
    el.id = id + '_' + idx;
    if (!el.dataset.poster) {
      el.dataset.poster = 'https://img.youtube.com/vi/' + id + '/sddefault.jpg';
    }
    // show in popup, other defined target or replace el with player
    var target = el.dataset.target || id + '_' + idx;

    if (el.dataset.autoplay) {
      load();
    }  else {
      // create placeholder background image
      if (el.dataset.poster) {
        el.style.backgroundImage = "url('" + el.dataset.poster + "')";
      }
      // show placeholder title
      if (el.title) {
        let title = document.createElement('div');
        title.className = 'title';
        title.appendChild(document.createTextNode(el.title));
        el.appendChild(title);
      }
      // show placeholder external message
      if (el.dataset.external) {
        externalMessage(el);
      }
      // show placeholder youtube like play button
      var play = document.createElement('div');
      play.className = 'play';
      el.appendChild(play);
    }
    // create popup
    function createPopup() {
      var overlay = document.createElement('div');
      overlay.className = 'overlay';
      document.body.appendChild(overlay);
      var content = document.createElement('div');
      content.className = 'popup-content';
      overlay.appendChild(content);
      var close = document.createElement('span');
      close.className = 'popup-close';
      overlay.appendChild(close);
      close.addEventListener('click', function (event) {
        player.stopVideo();
        document.body.removeChild(overlay);
      });
      return content;
    }
    // youtube player API
    function onPlayerReady(event) {
      event.target.playVideo();
    }
    function onPlayerStateChange(event) {
      if (event.data == window.YT.PlayerState.ENDED) {
        player.stopVideo();
        document.body.removeChild(overlay);
      }
    }
    // create youtube player
    function createPlayer() {
      window.YT.ready(function() {
        var dest, obj = {
          videoId: id,
          host: 'https://www.youtube-nocookie.com',
          events: {
            'onReady': onPlayerReady
          }
        };
        var sizes = [
          { width: '426', height: '240' },
          { width: '640', height: '360' },
          { width: '854', height: '480' },
          { width: '1280', height: '720' },
          { width: '1920', height: '1080' }
        ];
        if (target == 'popup') {
          let popup = createPopup();
          popup.id = 'popup-content';
          dest = 'popup-content';
          obj.events['onStateChange'] = onPlayerStateChange;
          for (var i = 0; i < sizes.length; i++) {
            if (sizes[i].width <= window.innerWidth) {
              obj.height = sizes[i].height;
              obj.width = sizes[i].width;
            } else {
              break;
            }
          }
        } else {
          dest = target;
          for (var i = 0; i < sizes.length; i++) {
            if (sizes[i].width >= el.offsetWidth) {
              obj.height = sizes[i].height;
              obj.width = sizes[i].width;
              break;
            }
          }
        }
        // create YouTube player
        player = new window.YT.Player(dest, obj);
      });
    }
    // load youtube external content
    function load() {
      // init once
      if (!document.getElementById('YouTubeApi')) {
        // load YouTube API
        var script = document.createElement('script');
        script.id = 'YouTubeApi';
        script.src = 'https://www.youtube.com/iframe_api';
        document.head.appendChild(script);
        script.addEventListener('load', createPlayer);
      } else {
        createPlayer();
      }
      if (target !== 'popup') {
        // remove placeholder click event
        el.removeEventListener('click', load);
      }
    }
    el.addEventListener('click', load);
  }
  // lazy loading vimeo
  function vimeo(el) {
    var player, id;
    // don't send referrer infos to external services
    if (!document.querySelector('meta[name="referrer"]')) {
      let meta = document.createElement('meta');
      meta.name = 'referrer';
      meta.content = 'no-referrer';
      document.head.appendChild(meta);
    }
    // get Vimeo video id
    if (el.dataset.src && !isNaN(el.dataset.src)) {
      id = el.dataset.src;
    } else {
      let src = el.dataset.src;
      let regExp = /http:\/\/(www\.)?vimeo.com\/(\d+)($|\/)/;
      let match = src.match(regExp);
      if (match){
        id = match[2];
      }
    }
    let idx = Math.floor(Math.random() * 6);
    el.id = id + '_' + idx;
    // show in popup, other defined target or replace el with player
    var target = el.dataset.target || id + '_' + idx;

    if (el.dataset.autoplay) {
      load();
    }  else {
      // create placeholder background image
      if (el.dataset.poster) {
        el.style.backgroundImage = "url('" + el.dataset.poster + "')";
      }
      // show placeholder title
      if (el.title) {
        let title = document.createElement('div');
        title.className = 'title';
        title.appendChild(document.createTextNode(el.title));
        el.appendChild(title);
      }
      // show placeholder external message
      if (el.dataset.external) {
        externalMessage(el);
      }
      // show placeholder youtube like play button
      var play = document.createElement('div');
      play.className = 'play';
      el.appendChild(play);
    }
    // create popup
    function createPopup() {
      var overlay = document.createElement('div');
      overlay.className = 'overlay';
      document.body.appendChild(overlay);
      var content = document.createElement('div');
      content.className = 'popup-content';
      overlay.appendChild(content);
      var close = document.createElement('span');
      close.className = 'popup-close';
      overlay.appendChild(close);
      close.addEventListener('click', function (event) {
        player.stopVideo();
        document.body.removeChild(overlay);
      });
      return content;
    }
    // load vimeo external content
    function load() {
      let iframe = document.createElement('iframe');
      iframe.src = 'https://player.vimeo.com/video/' + id + '?autoplay=1&autopause=0';
      iframe.dataset.external = 'Vimeo';
      iframe.classList.add('vimeo');
      var sizes = [
        { width: '426', height: '240' },
        { width: '640', height: '360' },
        { width: '854', height: '480' },
        { width: '1280', height: '720' },
        { width: '1920', height: '1080' }
      ];
      for (var i = 0; i < sizes.length; i++) {
        if (sizes[i].width >= el.offsetWidth) {
          iframe.height = sizes[i].height;
          iframe.width = sizes[i].width;
          break;
        }
      }
      if (el.dataset.width) {
        iframe.width = el.dataset.width;
      }
      if (el.dataset.height) {
        iframe.height = el.dataset.height;
      }
      iframe.style = el.style;
      el.parentNode.insertBefore(iframe, el);
      el.parentNode.removeChild(el);
    }
    el.addEventListener('click', load);
  }
   // lazy loading maps (OpenStreetMap)
  function loadMap(el) {
    function showMap(el) {
      let lat = el.dataset.lat;
      let lon = el.dataset.lon;
      let zoom = el.dataset.zoom;
      zoom = zoom || 18;
      if (lat && lon) {
        // initialize map
        var map = L.map(el, {scrollWheelZoom: false}).setView([lat, lon], zoom);
        // set map tiles source
        L.tileLayer('https://tile.openstreetmap.org/{z}/{x}/{y}.png', {
          attribution: 'Map data &copy; <a href="https://www.openstreetmap.org/" target="_blank">OpenStreetMap</a> contributors',
          maxZoom: 18,
          minZoom: 8
        }).addTo(map);
        el.style.backgroundImage = 'none';
        var LeafIcon = L.Icon.extend({
          options: {
            shadowUrl: 'code/OpenStreetMap/images/marker-shadow.png',
            iconSize:     [25, 41],
            shadowSize:   [41, 41],
            iconAnchor:   [12, 41],
            shadowAnchor: [12, 41],
            popupAnchor:  [1, -34]
          }
        });
        var blueIcon = new LeafIcon({iconUrl: 'code/OpenStreetMap/images/marker-icon-2x-blue.png'});
        var redIcon = new LeafIcon({iconUrl: 'code/OpenStreetMap/images/marker-icon-2x-red.png'});
        var blueWindIcon = new LeafIcon({iconUrl: 'code/OpenStreetMap/images/marker-icon-windmill-blue.png'});
        var redWindIcon = new LeafIcon({iconUrl: 'code/OpenStreetMap/images/marker-icon-windmill-red.png'});
        var blueWaterIcon = new LeafIcon({iconUrl: 'code/OpenStreetMap/images/marker-icon-watermill-blue.png'});
        var redWaterIcon = new LeafIcon({iconUrl: 'code/OpenStreetMap/images/marker-icon-watermill-red.png'});
        if (el.classList.contains('geo-map')) {
          map.on('map-container-resize', function () {
            map.invalidateSize(); // doesn't seem to do anything
          });
          function showGeo(e) {
            let lat  = e.latlng.lat.toFixed(5);
            let lon  = e.latlng.lng.toFixed(5);
            console.log('lat: ' + lat + "; lon: " + lon);
          }
          map.on('click', showGeo);
          let hits;
          if (el.dataset.selector) {
            hits = document.querySelectorAll(el.dataset.selector);
          } else {
            hits = document.querySelectorAll("section.geo-item, li.geo-data, li.geo-data-x");
          }
          for (let i=0; i < hits.length; i++) {
            let title_el, _title, _info, _lat, _lon;
            let info_el = hits[i].querySelector('.address');
            let link_el = hits[i].querySelector('span.links');
            let lat_el = hits[i].querySelector('.lat');
            let lon_el = hits[i].querySelector('.lon');
            if (hits[i].tagName.toLowerCase() == 'section') {
              title_el = hits[i].querySelector('h3');
              if (info_el) {
                _info = info_el.innerHTML;
                hits[i].id = 'c'+i;
              }
            } else {
              title_el = hits[i].querySelector('.title');
              if (info_el) {
                _info = info_el.innerText;
                _info = _info.replace(', ', '<br />');
              }
            }
            if (title_el) {
              _title = title_el.innerHTML;
            }
            if (link_el) {
              _info = _info + '<br />' + link_el.innerHTML;
            }
            if (lat_el) {
              _lat = lat_el.innerHTML;
            }
            if (lon_el) {
              _lon = lon_el.innerHTML;
            }
            // add marker to the map
            let marker;
            if (hits[i].classList.contains('geo-data')) {
              if (hits[i].classList.contains('wind')) {
                marker = L.marker([_lat, _lon], {icon: redWindIcon}).addTo(map);
              } else if (hits[i].classList.contains('water')) {
                marker = L.marker([_lat, _lon], {icon: redWaterIcon}).addTo(map);
              } else {
                marker = L.marker([_lat, _lon], {icon: redIcon}).addTo(map);
              }
            } else if (hits[i].classList.contains('geo-item')) {
              if (hits[i].classList.contains('wind')) {
                marker = L.marker([_lat, _lon], {icon: blueWindIcon}).addTo(map);
              } else if (hits[i].classList.contains('water')) {
                marker = L.marker([_lat, _lon], {icon: blueWaterIcon}).addTo(map);
              } else {
                marker = L.marker([_lat, _lon], {icon: blueIcon}).addTo(map);
              }
              hits[i].parentNode.setAttribute('id', 'c'+i);
              _info = _info + ', <a href="#c'+i+'">zum Eintrag</a>';
            } else {
              marker = L.marker([_lat, _lon], {icon: redIcon}).addTo(map);
            }
            // add popup to the marker
            marker.bindPopup("<strong>" + _title + "</strong><br />" + _info);
          }
          map.setView([lat, lon], zoom);
        } else {
          let parent, title_el, _title, _info;
          parent = el.parentNode;
          if (parent.tagName.toLowerCase() == 'section') {
            title_el = parent.querySelector('h3');
            if (title_el) {
              _title = title_el.innerHTML;
            }
          }
          let info_el = parent.querySelector('.address');
          if (info_el) {
            _info = info_el.innerHTML;
          }
          let link_el = parent.querySelector('span.links');
          if (link_el) {
            _info = _info + '<br />' + link_el.innerHTML;
          }
          if (_title || _info) {
            // add marker to the map
            let marker;
            if (parent.classList.contains('wind')) {
              marker = L.marker([lat, lon], {icon: blueWindIcon}).addTo(map);
            } else if (parent.classList.contains('water')) {
              marker = L.marker([lat, lon], {icon: blueWaterIcon}).addTo(map);
            } else {
              marker = L.marker([lat, lon], {icon: blueIcon}).addTo(map);
            }
            // add popup to the marker
            marker.bindPopup("<strong>" + _title + "</strong><br />" + _info);
            // center map at this marker
            map.panTo(new L.LatLng(lat, lon));
            function showGeo(e) {
              let lat  = e.latlng.lat.toFixed(5);
              let lon  = e.latlng.lng.toFixed(5);
              console.log('lat: ' + lat + "; lon: " + lon);
            }
            map.on('click', showGeo);
          }
        }
      }
    }
    // create placeholder background image
    if (el.dataset.poster) {
      el.style.backgroundImage = "url('" + el.dataset.poster + "')";
    }
    let external = document.createElement('div');
    external.className = 'external-text';
    let language = document.documentElement.lang || 'en';
    let copyright = document.createElement('div');
    copyright.className = 'copyright';
    copyright.innerHTML = 'Map data &copy; <a href="https://www.openstreetmap.org/" target="_blank">OpenStreetMap</a> contributors';
    el.appendChild(copyright);
    let txt;
    switch (language.substr(0,2)) {
      case 'de': txt = 'Lädt externe Inhalte von OpenStreetMap'; break;
      case 'fr': txt = 'Charger du contenu externe à partir de OpenStreetMap'; break;
      case 'it': txt = 'Carica contenuti esterni da OpenStreetMap'; break;
      case 'es': txt = 'Cargar contenido externo de OpenStreetMap'; break;
      default: txt = 'Load external Content from OpenStreetMap'; break;
    }
    external.appendChild(document.createTextNode(txt));
    el.appendChild(external);
    if (!el.classList.contains('geo-map')) {
      // play button
      var play = document.createElement('div');
      play.className = 'external-play';
      el.appendChild(play);
    }
    function loadMap(event) {
      external.classList.add('hidden');
      if (play) {
        play.classList.add('hidden');
      }
      showMap(el);
      el.removeEventListener('click', loadMap);
    }
    el.addEventListener('click', loadMap);
  }
  // lazy loading twitter
  function twitter(el) {
    if (el.dataset.href) {
      let a = document.createElement('a');
      a.classList.add("twitter-timeline");
      a.href = el.dataset.href;
      if (el.dataset.info) {
        a.appendChild(document.createTextNode(el.dataset.info));
      }
      el.appendChild(a);
    }
    if (el.dataset.autoload) {
      load();
    }  else {
      // create placeholder background image
      if (el.dataset.poster) {
        el.style.backgroundImage = "url('" + el.dataset.poster + "')";
      }
      // show placeholder title
      if (el.info) {
        let title = document.createElement('div');
        title.className = 'title';
        title.appendChild(document.createTextNode(el.info));
        el.appendChild(title);
      }
      // show placeholder external message
      if (el.dataset.external) {
        var msg = externalMessage(el);
      }
      // show placeholder twitter like bird
      var play = document.createElement('div');
      play.className = 'play';
      el.appendChild(play);
    }
    function load() {
      let script = document.createElement('script');
      script.src = "https://platform.twitter.com/widgets.js";
      script.setAttribute('charset', 'utf-8');
      script.async = true;
      el.appendChild(script);
      // remove placeholder click event
      el.removeEventListener('click', load);
      if (msg) {
        msg.classList.add('hidden');
      }
      play.classList.add('hidden');
    }
    el.addEventListener('click', load);
  }
  // viewport intersection
  function Intersection(els, callback) {
    // create intersection observer
    let lazyobserver = new IntersectionObserver(function(entries, observer) {
      for (var i=0; i < entries.length; i++) {
        if (entries[i].isIntersecting) {
          callback(entries[i].target);
          if (entries[i].target.tagName.toLowerCase() !== 'video') {
            lazyobserver.unobserve(entries[i].target);
          }
        }
      }
    }, {});
    // start observing
    for (var i=0; i < els.length; i++) {
      lazyobserver.observe(els[i]);
    }
  }
  // observe elements that used to be lazy loaded
  function observeElements(dom) {
    // activate viewport for image loading
    var images = dom.querySelectorAll("img[data-src]");
    for (var i=0; i < images.length; i++) {
      // don't load images lazy, that are in viewport
      let rect = images[i].getBoundingClientRect();
      if (rect.top <= window.innerHeight && rect.top >= -rect.height &&
          rect.left <= window.innerWidth && rect.left >= -rect.width &&
          !!(images[i].offsetWidth || images[i].offsetHeight || images[i].getClientRects().length)) {
        images[i].removeAttribute('loading');
        loadImg(images[i]);
      } else if (!images[i].getAttribute('loading')) {
        images[i].setAttribute('loading', 'lazy')
      }
    }
    images = dom.querySelectorAll("img[loading='lazy']");
    if (images.length > 0) {
      // load all images before print
      function beforeprint(event) {
        if ("loading" in HTMLImageElement.prototype) {
          for (var i=0; i < images.length; i++) {
            images[i].setAttribute('loading', 'eager');
          }
        } else {
          for (var i=0; i < images.length; i++) {
            loadImg(images[i]);
          }
        }
        function sleep(milliseconds) {
        const date = Date.now();
          let currentDate = null;
          do {
            currentDate = Date.now();
          } while (currentDate - date < milliseconds);
        }
        sleep(1000);
      }
      window.addEventListener("beforeprint", beforeprint);
      if (window.matchMedia) {
        var mediaQueryList = window.matchMedia('print');
        mediaQueryList.addListener(function (mql) {
          if (mql.matches) {
            beforeprint();
          }
        });
      }

      // use native lazy loading
      if ("loading" in HTMLImageElement.prototype) {
        // move attribute value from data-src to src
        console.log('viewport: Image lazy loading native');
        for (var i=0; i < images.length; i++) {
          loadImg(images[i]);
        }
      } else {
        // use IntersectionObserver
        console.log('viewport: Image lazy loading by IntersectionObserver');
        new Intersection(images, loadImg);
      }
    }

    // activate viewport for iframe loading
    var iframes = dom.querySelectorAll("iframe[data-src]");
    for (var i=0; i < iframes.length; i++) {
      if (!iframes[i].getAttribute('loading')) {
        iframes[i].setAttribute('loading', 'lazy')
      }
    }
    iframes = dom.querySelectorAll("iframe[loading='lazy']");
    if (iframes.length > 0) {
      if ('loading' in HTMLIFrameElement.prototype) {
        // native support, move attribute value from data-src to src
        console.log('viewport: IFrame lazy loading native');
        for (var i=0; i < iframes.length; i++) {
          loadIframe(iframes[i]);
        }
      } else {
        // use IntersectionObserver
        console.log('viewport: IFrame lazy loading by IntersectionObserver');
        new Intersection(iframes, loadIframe);
      }
    }

    // activate viewport for video loading
    var videos = dom.querySelectorAll("video[data-loading='lazy']");
    if (videos.length > 0) {
      console.log('viewport: Video lazy loading by IntersectionObserver');
      new Intersection(videos, loadVideo);
    }

    // activate viewport for youtube loading
    var media = dom.querySelectorAll('.youtube[data-src]');
    if (media.length > 0) {
      console.log('viewport: YouTube lazy loading by IntersectionObserver');
      new Intersection(media, youtube);
    }

    // activate viewport for vimeo loading
    var media = dom.querySelectorAll('.vimeo[data-src]');
    if (media.length > 0) {
      console.log('viewport: Vimeo lazy loading by IntersectionObserver');
      new Intersection(media, vimeo);
    }

    // activate viewport for external content
    var external = dom.querySelectorAll('div.external[data-src]');
    if (external.length > 0) {
      console.log('viewport: External lazy loading by IntersectionObserver');
      new Intersection(external, loadExternal);
    }

    // activate viewport for twitter loading
    var twttr = document.querySelectorAll('.twitter[data-href]');
    if (twttr.length > 0) {
      console.log('viewport: Twitter lazy loading by IntersectionObserver');
      new Intersection(twttr, twitter);
    }

    // activate viewport for background images
    var background = dom.querySelectorAll('[data-backgroundimage]');
    if (background.length > 0) {
      console.log('viewport: Background image lazy loading by IntersectionObserver');
      new Intersection(background, loadBackgroundImage);
    }

    // activate viewport for osm and geo-maps (OpenStreetMap)
    var maps = dom.querySelectorAll('.osm, .geo-map');
    if (maps.length > 0) {
      if (!document.getElementById('leaflet.css')) {
        var style = document.createElement('link');
        style.id = 'leaflet.css';
        style.type = 'text/css';
        style.rel = 'stylesheet';
        style.href = '/code/OpenStreetMap/leaflet.css';
        document.head.appendChild(style);
      }
      if (!document.getElementById('leaflet.js')) {
        var script = document.createElement('script');
        script.id = 'leaflet.js';
        script.src = '/code/OpenStreetMap/leaflet.min.js';
        script.addEventListener('load', initMaps);
        document.head.appendChild(script);
      } else {
        initMaps();
      }
      function initMaps() {
        console.log('viewport: Map lazy loading by IntersectionObserver');
        if (maps.length > 0) {
          new Intersection(maps, loadMap);
        }
      }
      // load all maps before print
      function beforeprintMap(event) {
        for (var i=0; i < maps.length; i++) {
          loadMap(maps[i])
        }
        function sleep(milliseconds) {
        const date = Date.now();
          let currentDate = null;
          do {
            currentDate = Date.now();
          } while (currentDate - date < milliseconds);
        }
        sleep(1000);
      }
      window.addEventListener("beforeprint", beforeprintMap);
      if (window.matchMedia) {
        var mediaQueryList = window.matchMedia('print');
        mediaQueryList.addListener(function (mql) {
          if (mql.matches) {
            beforeprintMap();
          }
        });
      }
    }
    // activate print for lazy loading poster images
    var poster = dom.querySelectorAll('[data-poster]');
    if (poster.length > 0) {
      // load all poster images before print
      function beforeprintPoster(event) {
        for (var i=0; i < poster.length; i++) {
          // create placeholder background image
          if (poster[i].dataset.poster) {
            poster[i].style.backgroundImage = "url('" + poster[i].dataset.poster + "')";
          }
        }
        function sleep(milliseconds) {
        const date = Date.now();
          let currentDate = null;
          do {
            currentDate = Date.now();
          } while (currentDate - date < milliseconds);
        }
        sleep(1000);
      }
      window.addEventListener("beforeprint", beforeprintPoster);
      if (window.matchMedia) {
        var mediaQueryList = window.matchMedia('print');
        mediaQueryList.addListener(function (mql) {
          if (mql.matches) {
            beforeprintPoster();
          }
        });
      }
    }

  }
  observeElements(document.body);

  // dom mutations
  function Mutation(els) {
    let mutationObserver = new MutationObserver(function(mutations, observer) {
      for (let o=0; o < mutations.length; o++) {
        let mutation = mutations[o];
        for (let i=0; i < mutation.addedNodes.length; i++) {
          let node = mutation.addedNodes[i];
          if (!node.tagName) {
            continue; // not an element
          }
          observeElements(node);
        }
      }
    });
    // start observing
    for (var i=0; i < els.length; i++) {
      mutationObserver.observe(els[i], { childList: true, subtree: true });
    }
  }
  var mutations = document.querySelectorAll("[data-mutation='true']");
  if (mutations.length > 0) {
    new Mutation(mutations);
  }

  // expose obeserve method globally
  window.observeElements = observeElements;
})();
